//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "ThreadWorker.h"
#include "ScopedLock.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------

//   Important: Methods and properties of objects in VCL can only be
//   used in a method called using Synchronize, for example:
//
//      Synchronize(&UpdateCaption);
//
//   where UpdateCaption could look like:
//
//      void __fastcall ThreadWorker::UpdateCaption()
//      {
//        Form1->Caption = "Updated in a thread";
//      }
//---------------------------------------------------------------------------

__fastcall ThreadWorker::ThreadWorker(bool CreateSuspended, bool &playSoundWhenDone)
	: TThread(CreateSuspended),
	playSoundWhenDone(playSoundWhenDone)
{
	state = IDLE;
}
//---------------------------------------------------------------------------
void __fastcall ThreadWorker::Execute()
{
	while (!Terminated)
	{
		switch (state)
		{
		case IDLE:
			Sleep(1000);
			break;
		case RUNNING:
			{
				ScopedLock<Mutex> lock(mutex);
				if (jobs.empty())
				{
					if (abort == false)
					{
						PlaySound("SystemAsterisk", NULL, SND_ASYNC);
					}
					state = IDLE;
					break;
				}
				Job &job = jobs.front();
				job.Execute(&abort, priority);				
				jobs.pop_front();
			}
			break;
		case STOPPING:
			state = IDLE;
			break;
		}
		if (abort)
		{
			state = IDLE;
			jobs.clear();
			break;
		}
	}
}
//---------------------------------------------------------------------------

void ThreadWorker::AddJob(Job& job)
{
	ScopedLock<Mutex> lock(mutex);
	jobs.push_back(job);
}

void ThreadWorker::Start(int priority)
{
	if (state == IDLE)
	{
		state = RUNNING;
		this->priority = priority;
		abort = false;
	}
}

void ThreadWorker::Stop(void)
{
	if (state == RUNNING)
		state = STOPPING;
}

